#include "mex.h"
#include <math.h>
#include "bitstream.h"
#include "bitplane.h"
#include "interleave.h"
#include "decoder.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	char *filename;
	int filename_len;
	FILE *fid;
	double bitrate,fmin,fmax;
	unsigned int minBlock,shift,binsperoctave;
	decoder *dec;
	int numSamples,numSlots,numBlocks;
	int output_len;
	double *output;
	int numFrames,numFreqs;
	int n,block,frame,freq;
	int bin,bin_f;
	double amp,f;
	mxArray *p,*temp;

	/* Check number of arguments */
	if (nrhs<2) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	fid = fopen(filename,"r");
	if (fid==NULL) mexErrMsgTxt( "Can't open file.");
	bitrate = mxGetPr(prhs[1])[0];
	if (bitrate<=0||bitrate>1024000) mexErrMsgTxt( "Bitrate must be in the range [0,1024000].");

	/* Parameters */
	if (nrhs>=3) {
		if (!mxIsStruct(prhs[2])) mexErrMsgTxt("Parameters must be a structure.");
		p = (mxArray*)prhs[2];
	} else {
		p = mxCreateStructMatrix(0,0,0,NULL);
	}
	minBlock = 7;
	temp = mxGetField(p, 0, "minBlock");
	if (temp!=NULL) minBlock = mxGetPr(temp)[0];
	shift = 8192;
	temp = mxGetField(p, 0, "hopsize");
	if (temp!=NULL) shift = (int)mxGetPr(temp)[0];
	binsperoctave = 36;
	temp = mxGetField(p, 0, "bpo");
	if (temp!=NULL) binsperoctave = (int)mxGetPr(temp)[0];
	fmin = 73.41619;
	temp = mxGetField(p, 0, "fmin");
	if (temp!=NULL) fmin = mxGetPr(temp)[0];
	fmax = fmin * 16;
	temp = mxGetField(p, 0, "fmax");
	if (temp!=NULL) fmax = mxGetPr(temp)[0];

	/* Init decoder */
	dec = new decoder( fid );
	if (dec->sampleRate!=44100) {
		delete( dec );
		mexErrMsgTxt( "Invalid file.");
	}
	numSamples = dec->numSamples;
	numSlots = dec->numSlots;
	numBlocks = dec->numBlocks;

	/* Init output */
	if (nlhs>0) {
		output_len = (numSamples-2*32768)/shift;
		plhs[0] = mxCreateDoubleMatrix(output_len,binsperoctave,mxREAL);
		output = mxGetPr(plhs[0]);
	}

	/* Decoding */
	for ( n=0 ; n<numSlots ; n++ ) {
		if (dec->decode_slot(bitrate)) {
			for ( block=minBlock-1 ; block<numBlocks ; block++ ) {
				numFrames = dec->get_numFrames(block);
				numFreqs = dec->get_numFreqs(block);
				for ( frame=0 ; frame<numFrames ; frame++ ) {
					bin = ((n*numFrames+frame)*numFreqs-32768+8192) / shift;
					if (bin>=0&&bin<output_len) {
						for ( freq=0 ; freq<numFreqs ; freq++ ) {
							f = ((double)freq+0.5)*22050.0/(double)numFreqs;
							if (f>=fmin&&f<=fmax) {
								bin_f = (int)round( (double)binsperoctave*log2(f/fmin) )%binsperoctave;
								amp = dec->get_coef(block,frame,freq);
								if (amp<0) amp = -amp;
								if (nlhs>0) output[bin_f*output_len+bin] += amp;
							}
						}
					}
				}
			}

		}
	}

	/* Clean */
	delete( dec );
	fclose( fid );
}

