#include "mex.h"
#include <math.h>
#include "bitstream.h"
#include "bitplane.h"
#include "interleave.h"
#include "decoder.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	char *filename;
	int filename_len;
	FILE *fid;
	double bitrate;
	double *output;
	decoder *dec;
	unsigned int n,numSlots;
	int block,frame,freq,pos,output_len;
	int numBlocks,numFrames,numFreqs;
	double scale = 1/sqrt(2);

	/* Check number of arguments */
	if (nrhs<2) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	fid = fopen(filename,"r");
	if (fid==NULL) mexErrMsgTxt( "Can't open file.");
	bitrate = mxGetPr(prhs[1])[0];
	if (bitrate<=0||bitrate>1024000) mexErrMsgTxt( "Bitrate must be in the range [0,1024000].");

	/* Init decoder */
	dec = new decoder( fid );
	if (dec->sampleRate!=44100) {
		delete( dec );
		mexErrMsgTxt( "Invalid file.");
	}
	numSlots = dec->numSlots;
	numBlocks = dec->numBlocks;

	/* Init output */
	if (nlhs>0) {
		output_len = (numSlots+1)*numBlocks*8192;
		plhs[0] = mxCreateDoubleMatrix(output_len,1,mxREAL);
		output = mxGetPr(plhs[0]);
	}

	/* Decoding */
	for ( n=0 ; n<numSlots ; n++ ) {
		if (dec->decode_slot(bitrate)&&nlhs>0) {
			for ( block=0 ; block<numBlocks ; block++ ) {
				numFrames = dec->get_numFrames(block);
				numFreqs = dec->get_numFreqs(block);
				for ( frame=0 ; frame<numFrames ; frame++ ) {
					for ( freq=0 ; freq<numFreqs ; freq++ ) {
						output[block*(numSlots+1)*8192+(n+1)*numFrames*numFreqs+frame*numFreqs+freq] = dec->get_coef(block,frame,freq)*scale;
					}
				}
			}
		}
	}

	/* Clean */
	delete( dec );
	fclose( fid );
}
