#include "mex.h"
#include <math.h>
#include "bitstream.h"
#include "bitplane.h"
#include "interleave.h"
#include "decoder.h"
#include "filters.h"
#include "dct.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	char *filename;
	int filename_len;
	FILE *fid;
	double bitrate,temp;
	decoder *dec;
	int numSamples,numSlots,numBlocks,totalcoefs;
	int output_len;
	double *output,*numcoefs;
	int numFrames,numFreqs;
	int v,u,x,y,n,m,block,frame,freq;
	int idx;
	double amp,f;
	double hist2d[8][40];
	double eps = 1e-16;

	/* Check number of arguments */
	if (nrhs<3) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	fid = fopen(filename,"r");
	if (fid==NULL) mexErrMsgTxt( "Can't open file.");
	bitrate = mxGetPr(prhs[1])[0];
	if (bitrate<=0||bitrate>1024000) mexErrMsgTxt( "Bitrate must be in the range [0,1024000].");
	if ((int)mxGetDimensions(prhs[2])[0]!=1||(int)mxGetDimensions(prhs[2])[1]!=8)
		mexErrMsgTxt( "Coefs number must be a row vector of length 8.");
	numcoefs = mxGetPr(prhs[2]);
	totalcoefs = 0;
	for (n=0;n<8;n++) totalcoefs += (int)numcoefs[n];

	/* Init decoder */
	dec = new decoder( fid );
	if (dec->sampleRate!=44100) {
		delete( dec );
		mexErrMsgTxt( "Invalid file.");
	}
	numSamples = dec->numSamples;
	numSlots = dec->numSlots;
	numBlocks = dec->numBlocks;

	/* Init output */
	if (nlhs>0) {
		output_len = (numSamples-2*32768)/8192;
		plhs[0] = mxCreateDoubleMatrix(output_len,totalcoefs,mxREAL);
		output = mxGetPr(plhs[0]);
	}

	/* Decoding */
	for ( n=0 ; n<numSlots ; n++ ) {
		if (dec->decode_slot(bitrate)) {
			for ( block=0 ; block<numBlocks ; block++ ) {
				numFrames = dec->get_numFrames(block);
				numFreqs = dec->get_numFreqs(block);
				for (m=0;m<40;m++) {
					hist2d[block][m] = 0;
					for ( frame=0 ; frame<numFrames ; frame++ ) {
						for ( freq=filters_first[block][m] ; freq<=filters_last[block][m] ; freq++ ) {
							amp = dec->get_coef(block,frame,freq);
							if (amp<0) amp = -amp;
							
							hist2d[block][m] += amp*filters_weight[block][m][freq-filters_first[block][m]];
						}
					}
					hist2d[block][m] = log10( hist2d[block][m] + eps );
				}
			}
			idx = n-3;
			if (idx>=0&&idx<output_len) {
				totalcoefs = 0;
				for (v=0;v<8;v++) {
					for (u=0;u<(int)numcoefs[v];u++) {
						temp = 0;
						for (y=0;y<8;y++) {
							for (x=0;x<40;x++) {
								temp += hist2d[y][x]*dct[v][u][y][x];
							}
						}
						if (nlhs>0) output[totalcoefs*output_len+idx] = temp;
						totalcoefs++;
					}
				}
			}
		}
	}

	/* Clean */
	delete( dec );
	fclose( fid );
}

