#include "mex.h"
#include <math.h>
#include "imdct.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	double *input,*output;
	long unsigned int input_len,output_len,n;
	int numBlocks,numFrames,numFreqs;
	int block,frame,freq,sample,pos;
	const int *input_dims;
	imdct *im[8];
	double buffer[16384];
	double scale = sqrt(2);

	/* Check number of arguments */
	if (nrhs<1) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	input_dims = mxGetDimensions(prhs[0]);
	if (input_dims[0]==1) {
		input_len = input_dims[1];
	} else {
		input_len = input_dims[0];
	}
	input = mxGetPr(prhs[0]);

	/* Init IMDCT */
	numBlocks = 8;
	for ( block=0 ; block<numBlocks ; block++ ) im[block] = new imdct( 128 << block );

	/* Init output */
	if (nlhs>0) {
		output_len = input_len/8-2*32768;
		plhs[0] = mxCreateDoubleMatrix(output_len,1,mxREAL);
		output = mxGetPr(plhs[0]);
	}

	/* Synthesis */
	for ( block=0 ; block<numBlocks ; block++ ) {
		numFreqs = 64 << block;
		numFrames = (input_len/8)/numFreqs;
		for ( frame=0 ; frame<numFrames ; frame++ ) {
			im[block]->compute(input+(input_len/8)*block+numFreqs*frame,buffer);
			if (nlhs>0) {
				pos = frame*numFreqs - (numFreqs>>1) - 32768;
				for ( sample=0 ; sample<(numFreqs<<1) ; sample++ ) {
					if (pos+sample>=0&&pos+sample<output_len) {
						output[pos+sample] += buffer[sample];
					}
				}
			}
		}
	}

	/* Clean */
	for ( block=0 ; block<numBlocks ; block++ ) delete( im[block] );
}

