/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include <string.h>
#include "bitstream.h"
#include "bitplane.h"

bitplane::bitplane( unsigned int set_data_len ) {

	data_len = set_data_len;
	sig = new unsigned char[data_len];
	level_max = (1<<30);
	bp_len = data_len;
}

bitplane::~bitplane( void ) {
	delete[]( sig );
}

void bitplane::decode( bitstream *bs, double *bufferOut, double greatest_amp, int data_max_nbits, int k_init) {

	/* Init output */
	data_dec = bufferOut;

	/* Decode max value */
	bs->get_unsigned(&data_max_q_u,data_max_nbits);
	data_max_q = (double) data_max_q_u * pow(2,-data_max_nbits) * greatest_amp;

	/* Init arrays */
	memset (sig,0,data_len);
	memset (data_dec,0,data_len*sizeof(double));

	/* Init level */
	level = 2;

	/* Init bitplane subset length */
	bp_len = data_len;

	/* Loop */
	while (true)  {

		/* Decode one bitplane */
		if ( bitplane_decode(bs, k_init) ) break;

		/* Update the level */
		level = level << 1;
		if (level>=level_max) break;
	}
}

int bitplane::bitplane_decode( bitstream *bs, int k_init ) {

	/* Declarations */
	unsigned int runlen[data_len],signs[data_len];
	unsigned int num_ones, n, current_pos, last_pos, cum, u;
	long int num_zeros,k,k2;
	int ret,bit,stop,ref_bit;

	/* Init return */
	ret = 0;

	/* Decode run-lengths and signs */
	cum = 0;
	num_ones = 0;
	k = k_init;
	k2 = 1<<k_init;
	while ( true ) {
		num_zeros = 0;
		while (true) {
			if ( bs->get_bit(&bit) ) {ret = 1; goto End;}
			if (bit==0) break;
			num_zeros = num_zeros + k2;
			cum = cum + k2;
			if ( cum>bp_len) goto End;
			k++;k2 = k2<<1;
		}
		if ( bs->get_unsigned(&u,k) ) {ret = 1; goto End;}
		num_zeros = num_zeros + u;
		cum = cum + u + 1;
		if (k>0) {k--; k2=k2>>1;}
		if ( bs->get_bit(&bit) ) {ret = 1; goto End;}
		runlen[num_ones] = num_zeros;
		signs[num_ones] = (unsigned int)bit;
		num_ones++;
	}
	End:
	runlen[num_ones] = data_len;

	/* Set new significant coefficients + Refinement pass */
	current_pos = 0;
	last_pos = 0;
	num_ones = 0;
	bp_len = 0;
	stop = ret;
	for ( n=0 ; n<data_len ; n++ ) {
		if ( sig[n]==0 ) {
			current_pos++;
			if ( current_pos-last_pos-1==runlen[num_ones] ) {
				new_coef( n, signs[num_ones] );
				last_pos = current_pos;
				num_ones++;
				sig[n] = 1;
			} else {
				bp_len++;
			}
		} else {
			if (stop==0) {
				if ( bs->get_bit(&ref_bit) ) {
					stop=1;ret=1;
				} else {
					refine_coef( n, ref_bit );
				}
			}
		}
	}

	return ret;
}

void bitplane::new_coef( unsigned int n, unsigned int sign ) {

	data_dec[n] = 1.5 * data_max_q / level;
	if (sign==0) data_dec[n] = -data_dec[n];
}

void bitplane::refine_coef( unsigned int n, unsigned int ref_bit ) {

	unsigned int sign;
	double amp,tmp;

	amp = data_dec[n];
	sign = 1;
	if (amp<0) { amp = -amp; sign = 0; }
	tmp = data_max_q / (2*level);
	if (ref_bit==0) tmp = -tmp;
	data_dec[n] =  amp + tmp;
	if (sign==0) data_dec[n] = -data_dec[n];
}

